
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_MIPS_NONE,                0)
ELF_RELOC(R_MIPS_16,                  1)
ELF_RELOC(R_MIPS_32,                  2)
ELF_RELOC(R_MIPS_REL32,               3)
ELF_RELOC(R_MIPS_26,                  4)
ELF_RELOC(R_MIPS_HI16,                5)
ELF_RELOC(R_MIPS_LO16,                6)
ELF_RELOC(R_MIPS_GPREL16,             7)
ELF_RELOC(R_MIPS_LITERAL,             8)
ELF_RELOC(R_MIPS_GOT16,               9)
ELF_RELOC(R_MIPS_PC16,               10)
ELF_RELOC(R_MIPS_CALL16,             11)
ELF_RELOC(R_MIPS_GPREL32,            12)
ELF_RELOC(R_MIPS_UNUSED1,            13)
ELF_RELOC(R_MIPS_UNUSED2,            14)
ELF_RELOC(R_MIPS_UNUSED3,            15)
ELF_RELOC(R_MIPS_SHIFT5,             16)
ELF_RELOC(R_MIPS_SHIFT6,             17)
ELF_RELOC(R_MIPS_64,                 18)
ELF_RELOC(R_MIPS_GOT_DISP,           19)
ELF_RELOC(R_MIPS_GOT_PAGE,           20)
ELF_RELOC(R_MIPS_GOT_OFST,           21)
ELF_RELOC(R_MIPS_GOT_HI16,           22)
ELF_RELOC(R_MIPS_GOT_LO16,           23)
ELF_RELOC(R_MIPS_SUB,                24)
ELF_RELOC(R_MIPS_INSERT_A,           25)
ELF_RELOC(R_MIPS_INSERT_B,           26)
ELF_RELOC(R_MIPS_DELETE,             27)
ELF_RELOC(R_MIPS_HIGHER,             28)
ELF_RELOC(R_MIPS_HIGHEST,            29)
ELF_RELOC(R_MIPS_CALL_HI16,          30)
ELF_RELOC(R_MIPS_CALL_LO16,          31)
ELF_RELOC(R_MIPS_SCN_DISP,           32)
ELF_RELOC(R_MIPS_REL16,              33)
ELF_RELOC(R_MIPS_ADD_IMMEDIATE,      34)
ELF_RELOC(R_MIPS_PJUMP,              35)
ELF_RELOC(R_MIPS_RELGOT,             36)
ELF_RELOC(R_MIPS_JALR,               37)
ELF_RELOC(R_MIPS_TLS_DTPMOD32,       38)
ELF_RELOC(R_MIPS_TLS_DTPREL32,       39)
ELF_RELOC(R_MIPS_TLS_DTPMOD64,       40)
ELF_RELOC(R_MIPS_TLS_DTPREL64,       41)
ELF_RELOC(R_MIPS_TLS_GD,             42)
ELF_RELOC(R_MIPS_TLS_LDM,            43)
ELF_RELOC(R_MIPS_TLS_DTPREL_HI16,    44)
ELF_RELOC(R_MIPS_TLS_DTPREL_LO16,    45)
ELF_RELOC(R_MIPS_TLS_GOTTPREL,       46)
ELF_RELOC(R_MIPS_TLS_TPREL32,        47)
ELF_RELOC(R_MIPS_TLS_TPREL64,        48)
ELF_RELOC(R_MIPS_TLS_TPREL_HI16,     49)
ELF_RELOC(R_MIPS_TLS_TPREL_LO16,     50)
ELF_RELOC(R_MIPS_GLOB_DAT,           51)
ELF_RELOC(R_MIPS_PC21_S2,            60)
ELF_RELOC(R_MIPS_PC26_S2,            61)
ELF_RELOC(R_MIPS_PC18_S3,            62)
ELF_RELOC(R_MIPS_PC19_S2,            63)
ELF_RELOC(R_MIPS_PCHI16,             64)
ELF_RELOC(R_MIPS_PCLO16,             65)
ELF_RELOC(R_MIPS16_26,               100)
ELF_RELOC(R_MIPS16_GPREL,            101)
ELF_RELOC(R_MIPS16_GOT16,            102)
ELF_RELOC(R_MIPS16_CALL16,           103)
ELF_RELOC(R_MIPS16_HI16,             104)
ELF_RELOC(R_MIPS16_LO16,             105)
ELF_RELOC(R_MIPS16_TLS_GD,           106)
ELF_RELOC(R_MIPS16_TLS_LDM,          107)
ELF_RELOC(R_MIPS16_TLS_DTPREL_HI16,  108)
ELF_RELOC(R_MIPS16_TLS_DTPREL_LO16,  109)
ELF_RELOC(R_MIPS16_TLS_GOTTPREL,     110)
ELF_RELOC(R_MIPS16_TLS_TPREL_HI16,   111)
ELF_RELOC(R_MIPS16_TLS_TPREL_LO16,   112)
ELF_RELOC(R_MIPS_COPY,               126)
ELF_RELOC(R_MIPS_JUMP_SLOT,          127)
ELF_RELOC(R_MICROMIPS_26_S1,         133)
ELF_RELOC(R_MICROMIPS_HI16,          134)
ELF_RELOC(R_MICROMIPS_LO16,          135)
ELF_RELOC(R_MICROMIPS_GPREL16,       136)
ELF_RELOC(R_MICROMIPS_LITERAL,       137)
ELF_RELOC(R_MICROMIPS_GOT16,         138)
ELF_RELOC(R_MICROMIPS_PC7_S1,        139)
ELF_RELOC(R_MICROMIPS_PC10_S1,       140)
ELF_RELOC(R_MICROMIPS_PC16_S1,       141)
ELF_RELOC(R_MICROMIPS_CALL16,        142)
ELF_RELOC(R_MICROMIPS_GOT_DISP,      145)
ELF_RELOC(R_MICROMIPS_GOT_PAGE,      146)
ELF_RELOC(R_MICROMIPS_GOT_OFST,      147)
ELF_RELOC(R_MICROMIPS_GOT_HI16,      148)
ELF_RELOC(R_MICROMIPS_GOT_LO16,      149)
ELF_RELOC(R_MICROMIPS_SUB,           150)
ELF_RELOC(R_MICROMIPS_HIGHER,        151)
ELF_RELOC(R_MICROMIPS_HIGHEST,       152)
ELF_RELOC(R_MICROMIPS_CALL_HI16,     153)
ELF_RELOC(R_MICROMIPS_CALL_LO16,     154)
ELF_RELOC(R_MICROMIPS_SCN_DISP,      155)
ELF_RELOC(R_MICROMIPS_JALR,          156)
ELF_RELOC(R_MICROMIPS_HI0_LO16,      157)
ELF_RELOC(R_MICROMIPS_TLS_GD,           162)
ELF_RELOC(R_MICROMIPS_TLS_LDM,          163)
ELF_RELOC(R_MICROMIPS_TLS_DTPREL_HI16,  164)
ELF_RELOC(R_MICROMIPS_TLS_DTPREL_LO16,  165)
ELF_RELOC(R_MICROMIPS_TLS_GOTTPREL,     166)
ELF_RELOC(R_MICROMIPS_TLS_TPREL_HI16,   169)
ELF_RELOC(R_MICROMIPS_TLS_TPREL_LO16,   170)
ELF_RELOC(R_MICROMIPS_GPREL7_S2,        172)
ELF_RELOC(R_MICROMIPS_PC23_S2,          173)
ELF_RELOC(R_MICROMIPS_PC21_S2,          174)
ELF_RELOC(R_MICROMIPS_PC26_S2,          175)
ELF_RELOC(R_MICROMIPS_PC18_S3,          176)
ELF_RELOC(R_MICROMIPS_PC19_S2,          177)
ELF_RELOC(R_MIPS_NUM,                218)
ELF_RELOC(R_MIPS_PC32,               248)
ELF_RELOC(R_MIPS_EH,                 249)
